import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

name = "jarvisub"
version = "1.3-beta"
author = "Spidy"
author_email = "me@sppidy.me"
description = "A Secure and Powerful Python-Telethon Based Library For Ultroid Userbot."
license = "GNU AFFERO GENERAL PUBLIC LICENSE (v3)"
url = "https://github.com/Jarvis-Works/JarvisUserbot"

setuptools.setup(
    name=name,
    version=version,
    author=author,
    author_email=author_email,
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=url,
    license=license,
    packages=setuptools.find_packages(),
    install_requires=[
        "telethon>=1.19.5",
        "redis",
        "python-decouple==3.3",
        "cryptg",
        "nospamplus",
        "python-dotenv==0.15.0",
        "requests>=2.18.4",
        "telethon==1.19.1",
        "Pillow>=5.3.0",
        "PyGithub",
        "aiofiles",
        "aiohttp>=3.6.2",
        "async_generator",
        "beautifulsoup4>=4.9.1",
        "bs4",
        "bwb==3.0.0",
        "cfscrape",
        "coffeehouse",
        "cowpy",
        "cryptg",
        "dnspython",
        "emoji",
        "gTTS-token>=1.1.3",
        "gTTS>=2.0.1",
        "geopy",
        "gitpython",
        "google-api-python-client==1.12.8",
        "google-auth-oauthlib",
        "google_images_download>=2.7.1",
        "googletrans==4.0.0rc1",
        "gsearch",
        "hachoir",
        "heroku3",
        "httplib2",
        "humanize",
        "lxml>=4.5.2",
        "lyricsgenius",
        "oauth2client",
        "PyLyrics",
        "PyPDF2",
        "pySmartDL",
        "pybase64>=0.4.0",
        "pyfiglet",
        "pylast",
        "pymongo",
        "python-barcode",
        "python-dotenv",
        "python-magic",
        "pytube",
        "pytz",
        "qrcode",
        "regex",
        "requests>=2.18.4",
        "search-engine-parser>=0.4.2",
        "selenium",
        "speedtest-cli>=2.0.2",
        "sqlalchemy>=1.2",
        "telegraph",
        "tswift",
        "urbandict>=0.5",
        "wikipedia>=1.4.0",
        "youtube-dl",
        "covid",
        "validators",
        "psycopg2-binary",
        "pyrogram",
        "lottie",
        "glitch-this",
        "psutil",
        "pytube",
        "nospamplus",
        "apscheduler",

    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
